<?php
/*
Plugin Name: Adult Signature for UPS Residential Shipment
Description: Add-on plugin to ensure that whenever an address is recognized as 'Residential address' by UPS API, then by default "Delivery confirmation- Adult Signature" will be mandatory for that shipment.
Version: 1.0
Author: PluginHive
Author URI: https://www.pluginhive.com/about/
Text Domain: ph-residential-order-signature
*/

if ( ! defined( 'ABSPATH' ) ) {
		exit; // Exit if accessed directly.
}	

//check if UPS_WooCommerce_Shipping exists
if ( !class_exists( 'UPS_WooCommerce_Shipping' ) ) {   
	add_action( 'admin_init', 'ph_residential_order_signature_plugin_deactivate' );
	if ( ! function_exists( 'ph_residential_order_signature_plugin_deactivate' ) ) {
		function ph_residential_order_signature_plugin_deactivate() {
			  	if ( !class_exists( 'UPS_WooCommerce_Shipping' ) ){
		           deactivate_plugins( plugin_basename( __FILE__ ) );
		           wp_safe_redirect( admin_url('plugins.php') );    
			  	}
		}
	}
}

if( ! function_exists('ph_ups_signature_activate') ) {
	function ph_ups_residential_order_signature_activate(){

		if ( !is_plugin_active('ups-woocommerce-shipping/ups-woocommerce-shipping.php') ){
			deactivate_plugins( basename( __FILE__ ) );
			wp_die( __("Oops! You tried installing the Adult Signature for Residential Order Plugin without activating UPS WooCommerce Shipping Plugin. Kindly activate <b>UPS WooCommerce Shipping Plugin</b> and then try again", "ph_residential_order_signature" ), "", array('back_link' => 1 ));
		}	
	}
}
register_activation_hook( __FILE__, 'ph_ups_residential_order_signature_activate' );

const ADDRESS_CLASSIFICATION_VALUE_FOR_RESIDENTIAL = 2;
const DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE = 3;
const INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE = 2;

//For RATE REQUEST: Set Adult Signature required if Address validation response points to Residential Address
if( ! function_exists('ph_ups_signature_customization') ) {
	function ph_ups_signature_customization( $request, $package ) {
		
		$ups_settings 			= get_option( 'woocommerce_'.WF_UPS_ID.'_settings', null );

		if( ! class_exists('Ph_Ups_Address_Validation') ) {
			require_once dirname( dirname(__FILE__)) . '/ups-woocommerce-shipping/includes/class-ph-ups-address-validation.php';
		}
		$Ph_Ups_Address_Validation = new Ph_Ups_Address_Validation( $package['destination'], $ups_settings );

		$address_classification = $Ph_Ups_Address_Validation->residential_check;//if address is residential, $address_classification = 2

		//return request without change if address is not residential
		if ( isset($address_classification) && ($address_classification != ADDRESS_CLASSIFICATION_VALUE_FOR_RESIDENTIAL ) ){
			return $request;
		}

		//Get origin address from ups settings
		$origin_country_state = explode( ':', $ups_settings['origin_country_state'] );
		$origin_country = $origin_country_state[0];

		//check if shippment is international or not
		$international_shippment = false;
		if( $origin_country != $package['destination']['country'] ){
			$international_shippment = true;
		}

		//Adding Adult required node to request
		$request_delivery_node = explode("<DeliveryConfirmation>", $request);
		if( ! isset( $request_delivery_node[1] )) {
			if( $international_shippment ) {
				$request_shippment_service_node = explode("/<ShipmentServiceOptions>", $request);
				if( ! isset( $request_shippment_service_node[1] )) {
					$request_shippment_node = explode("</Shipment>", $request);
					$request = $request_shippment_node[0] . '<ShipmentServiceOptions>' . '<DeliveryConfirmation>' . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</ShipmentServiceOptions>' . "</Shipment>" . $request_shippment_node[1];
				}
				else {
					$request = $request_shippment_service_node[0] . '<DeliveryConfirmation>' . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</ShipmentServiceOptions>' . $request_shippment_service_node[1];
				}
			}
			else {
				$request_package_service_node = explode("</PackageServiceOptions>", $request);
				if( ! isset( $request_package_service_node[1] )) {
					$request_package_node = explode("</Package>", $request);
					$request = $request_package_node[0];
					for ($i = 1; $i < count($request_package_node); $i++){
						$request .= '<PackageServiceOptions>' . '<DeliveryConfirmation>' . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</PackageServiceOptions>' . "</Package>";
						$request .= $request_package_node[$i];
					}
				}
				else {
					$request = $request_package_service_node[0];
					for ($i = 1; $i < count($request_package_service_node); $i++){
						$request .= '<DeliveryConfirmation>' . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</PackageServiceOptions>';
						$request .= $request_package_service_node[$i];
					}
				}
			}
		}
		else {
			$request_DCISType_node = explode("</DCISType>", $request_delivery_node[1]);
			if( $international_shippment ) {
				$request = $request_delivery_node[0] . "<DeliveryConfirmation>" . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . $request_DCISType_node[1];
			}
			else {
				$request = $request_delivery_node[0] . "<DeliveryConfirmation>" . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . $request_DCISType_node[1];
			}
		}
		return $request;
	}
}
add_filter( 'wf_ups_rate_request', 'ph_ups_signature_customization' ,10,2);


//Send shipment confirm request with Adult Signature if Address validation response is Residential Address
if( ! function_exists('ph_ups_admin_signature_customization') ) {
	function ph_ups_admin_signature_customization( $request, $order, $shipment ) {

		$ups_settings 			= get_option( 'woocommerce_'.WF_UPS_ID.'_settings', null );
		
		if( ! class_exists('Ph_Ups_Address_Validation') ) {
			require_once dirname( dirname(__FILE__)) . '/ups-woocommerce-shipping/includes/class-ph-ups-address-validation.php';
		}

		$order_address = array(
			'country' => $order->shipping_country,
			'state' => $order->shipping_state,
			'postcode' => $order->shipping_postcode,
			'city' => $order->shipping_city,
			'address' => $order->shipping_address_1,
			'address_1' => $order->shipping_address_1,
			'address_2' => $order->shipping_address_2,
		);
		
		$Ph_Ups_Address_Validation = new Ph_Ups_Address_Validation( $order_address, $ups_settings );
		$address_classification = $Ph_Ups_Address_Validation->residential_check;

		//return request without change if address is not residential
		if ( isset($address_classification) && ($address_classification != ADDRESS_CLASSIFICATION_VALUE_FOR_RESIDENTIAL ) ){
			return $request;
		}

		$origin_country_state = explode( ':', $ups_settings['origin_country_state'] );
		$origin_country = $origin_country_state[0];

		//check if shippment is international or not
		$international_shippment = false;
		if( $origin_country != $order_address['country'] ){
			$international_shippment = true;
		}

		//Adding Adult required node to request
		$request_delivery_node = explode("<DeliveryConfirmation>", $request);
		if( ! isset( $request_delivery_node[1] )) {
			if( $international_shippment ) {
				$request_shippment_service_node = explode("/<ShipmentServiceOptions>", $request);
				if( ! isset( $request_shippment_service_node[1] )) {
					$request_shippment_node = explode("</Shipment>", $request);
					$request = $request_shippment_node[0] . '<ShipmentServiceOptions>' . '<DeliveryConfirmation>' . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</ShipmentServiceOptions>' . "</Shipment>" . $request_shippment_node[1];
				}
				else {
					$request = $request_shippment_service_node[0] . '<DeliveryConfirmation>' . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</ShipmentServiceOptions>' . $request_shippment_service_node[1];
				}
			}
			else {
				$request_package_service_node = explode("</PackageServiceOptions>", $request);
				if( ! isset( $request_package_service_node[1] )) {
					$request_package_node = explode("</Package>", $request);
					$request = $request_package_node[0];
					for ($i = 1; $i < count($request_package_node); $i++){
						$request .= '<PackageServiceOptions>' . '<DeliveryConfirmation>' . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</PackageServiceOptions>' . "</Package>";
						$request .= $request_package_node[$i];
					}
				}
				else {
					$request = $request_package_service_node[0];
					for ($i = 1; $i < count($request_package_service_node); $i++){
						$request .= '<DeliveryConfirmation>' . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . '</DeliveryConfirmation>' . '</PackageServiceOptions>';
						$request .= $request_package_service_node[$i];
					}
				}
			}
		}
		else {
			$request_DCISType_node = explode("</DCISType>", $request_delivery_node[1]);
			if( $international_shippment ) {
				$request = $request_delivery_node[0] . "<DeliveryConfirmation>" . '<DCISType>' . INTERNATIONAL_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . $request_DCISType_node[1];
			}
			else {
				$request = $request_delivery_node[0] . "<DeliveryConfirmation>" . '<DCISType>' . DOMESTIC_ADULT_SIGNATURE_REQUIRED_VALUE . '</DCISType>' . $request_DCISType_node[1];
			}
		}
		return $request;
	}
}

add_filter('wf_ups_shipment_confirm_request', 'ph_ups_admin_signature_customization', 10,3 );